classdef UDP < xcp.transportlayer.Base
% UDP Connection via a UDP socket.

% Copyright 2017-2018 The MathWorks, Inc.
    
properties (SetAccess = 'private')
    AddressString
    Port
    RxCounter
    Transport
end

properties (Hidden = true)
    TxCounter
end


methods
    
    function obj = UDP(addressString, port)
        % Create a UDP connection.
        obj.AddressString = addressString;
        obj.Port = port;
        obj.TxCounter = uint16(0);
        obj.RxCounter = uint16(0);
        obj.Transport  = clib.libxcp_transport_udp.Transport_udp.instance();       
        obj.Transport.setup(obj.AddressString,obj.Port,0, 'UDP', 1, 1);
    end
    
    function start(obj)
        % Check for the UDP connection already being open.
            
        if obj.Running
            error(message('vnt:Channel:ChannelAlreadyOnline'));
        end
        
        % Open the connection.
        if strcmpi(obj.Transport.ConnectionStatus, 'Connected')
            obj.Transport.disconnect();
        end
        obj.Transport.connect();       
        obj.Transport.flushInput();
        obj.Transport.flushOutput();
        obj.Running = true;
    end
    
    function stop(obj)
        % Close the connection.
        if obj.Running
            obj.Transport.disconnect();
        end
        obj.Running = false;
    end
     
    function transmit(obj, data)
        % Manage the transmit counter.
        frame = [typecast(uint16(length(data)),'uint8') typecast(uint16(obj.TxCounter),'uint8') data];
        if obj.TxCounter == uint16(65535)
            obj.TxCounter = uint16(0);
        else
            obj.TxCounter = obj.TxCounter+1;
        end
        
        % Send the new frame.
      obj.Transport.writeFrame(frame)
    end
    
    function data = receive(obj, maxCount) %#ok<INUSD>
        % Initialize the output as empty.
        data =[];
        
        done = false;
        while  ((done == false) && (obj.Running))
            [ errCode, rdata ] = obj.Transport.getReceivedResponseFromQueue(0);
          tdata = zeros(rdata.len, "uint8");
            if (errCode == 0 ) && (rdata.len > 0 )
                for ii =1:rdata.len
                   %display(ii);
                  %display(rdata.len)
                   tdata(ii)= uint8(rdata.frame(ii));
                end
                data = tdata;
                done=true;
            else
                if ( errCode ~= 0 )
                    done =true;
                %    display(errCode);
                end     
            end
        end   
    end
    function delete(obj)

        obj.Transport.del_receiveThread();
    end
end
end
