classdef (Hidden) Utility
% Utility Class that implement utility methods for XCP.

% Copyright 2012-2021 The MathWorks, Inc.

properties (Constant)
    % BYTES_IN_BYTE - The number of bytes in a byte.
    BYTES_IN_BYTE = 1;
    % BYTES_IN_WORD - The number of bytes in a word.
    BYTES_IN_WORD = 2;
    % BYTES_IN_DWORD - The number of bytes in a double word.
    BYTES_IN_DWORD = 4;
    % BYTES_IN_DLONG - The number of bytes in a double long.
    BYTES_IN_DLONG = 8;
    
    % CODE_FOR_POSITIVE_RESPONSE - Code for a positive response to an XCP command.
    CODE_FOR_POSITIVE_RESPONSE = 0xFFu8;
    % CODE_FOR_NEGATIVE_RESPONSE - Code for a negative response to an XCP command.
    CODE_FOR_NEGATIVE_RESPONSE = 0xFEu8;
    % CODE_FOR_SERVER_EVENT - Code for an XCP server event message.
    CODE_FOR_SERVER_EVENT = 0xFDu8;
    % CODE_FOR_SERVER_SERVICE_REQUEST - Code for an XCP service request message.
    CODE_FOR_SERVER_SERVICE_REQUEST = 0xFCu8;
    
    % RESOURCE_CAL - Resource bit mask for calibration.
    RESOURCE_CAL = 0x01u8;
    % RESOURCE_DAQ - Resource bit mask for DAQ.
    RESOURCE_DAQ = 0x04u8;
    % RESOURCE_STIM - Resource bit mask for STIM.
    RESOURCE_STIM = 0x08u8;
    % RESOURCE_PGM - Resource bit mask for programming.
    RESOURCE_PGM = 0x10u8;
    
    % NORMAL_MODE - Indicates for the CONNECT command to use normal mode.
    NORMAL_MODE = 0x00u8;
    % USER_DEFINED - Indicate for the CONNECT command to use user defined  mode.
    USER_DEFINED = 0x01u8;	
    
    % FIRST_ODT_ENTRY - Indicates for the SET_DAQ_PTR command the numeric
    %   value of the first ODT entry for the list.
    FIRST_ODT_ENTRY = 0x00u8;
    
    % IGNORE_BIT_OFFSET - Indicates for the WRITE_DAQ command to ignore bit
    %   offsets for a measurement.
    IGNORE_BIT_OFFSET = 0xFFu8;
    
    % NO_TRANSMISSION_RATE_PRESCALER - Indicates to the SET_DAQ_LIST_MODE
    %   command to not apply any prescaler to DAQ transmissions.
    NO_TRANSMISSION_RATE_PRESCALER = 0x01u8;
    % LET_SERVER_MANAGE_PRIORITY - Indicates to the SET_DAQ_LIST_MODE
    %   command to allow the server to manage list priority itself.
    LET_SERVER_MANAGE_PRIORITY = 0x00u8;
    
    % SET_SYNCHRONIZED_START_STOP - Indicates to the START_STOP_DAQ_LIST
    % command to start the measurement lists together.
    SET_SYNCHRONIZED_START_STOP = 0x02u8;
    
    % FIRST_PART_OF_SEED - Indication for the GET_SEED command that this is
    %   the first part of the seed value being sent.
    FIRST_PART_OF_SEED = 0x00u8;
    % REMAINING_PART_OF_SEED - Indication for the GET_SEED command that
    %   this is the additional seed value bytes being sent.
    REMAINING_PART_OF_SEED = 0x01u8;
    
    % SYNCH_STOP_ALL_LISTS - Indication for the START_STOP_SYNCH command to
    %   stop all lists.
    SYNCH_STOP_ALL_LISTS = 0x00u8;
    % SYNCH_START_SELECTED_LISTS - Indication for the START_STOP_SYNCH 
    %   command to start only selected lists.
    SYNCH_START_SELECTED_LISTS = 0x01u8;
    % SYNCH_STOP_SELECTED_LISTS - Indication for the START_STOP_SYNCH 
    %   command to stop only selected lists.
    SYNCH_STOP_SELECTED_LISTS = 0x02u8;
    
    % MAX_KEY_BYTES_PER_CTO - The number of key bytes that fit into a single
    %   CTO message.
    MAX_KEY_BYTES_PER_CTO = 6;
    
end

% Seed and key function response codes.
properties (Constant)
    SEED_KEY_SUCCESS = 0;
    SEED_KEY_ERROR_REQUESTED_PRIVILEGE_NOT_AVAILABLE = 1;
    SEED_KEY_ERROR_INVALID_SEED_LENGTH = 2;
    SEED_KEY_ERROR_INSUFFICIENT_KEY_LENGTH = 3;
    SEED_KEY_ERROR_LOAD_LIBRARY_FAILED = 4;
    SEED_KEY_ERROR_FUNCTION_NOT_FOUND = 5;
    
    MAP_SEED_KEY_RESPONSE_TO_NAME = containers.Map( ...
        { 0, 1, 2, 3, 4, 5 }, ...
        { 'SEED_KEY_SUCCESS', ...
        'SEED_KEY_ERROR_REQUESTED_PRIVILEGE_NOT_AVAILABLE', ...
        'SEED_KEY_ERROR_INVALID_SEED_LENGTH', ...
        'SEED_KEY_ERROR_INSUFFICIENT_KEY_LENGTH', ...
        'SEED_KEY_ERROR_LOAD_LIBRARY_FAILED', ...
        'SEED_KEY_ERROR_FUNCTION_NOT_FOUND' } );
end

% XCP error response code properties.
properties (Constant)
    ERR_CMD_SYNCH = 0x00u8;
    ERR_CMD_BUSY = 0x10u8;
    ERR_DAQ_ACTIVE = 0x11u8;
    ERR_PGM_ACTIVE = 0x12u8;
    ERR_CMD_UNKNOWN = 0x20u8;
    ERR_CMD_SYNTAX = 0x21u8;
    ERR_OUT_OF_RANGE = 0x22u8;
    ERR_WRITE_PROTECTED = 0x23u8;
    ERR_ACCESS_DENIED = 0x24u8;
    ERR_ACCESS_LOCKED = 0x25u8;
    ERR_PAGE_NOT_VALID = 0x26u8;
    ERR_MODE_NOT_VALID = 0x27u8;
    ERR_SEGMENT_NOT_VALID = 0x28u8;
    ERR_SEQUENCE = 0x29u8;
    ERR_DAQ_CONFIG = 0x2Au8;
    ERR_MEMORY_OVERFLOW = 0x30u8;
    ERR_GENERIC = 0x31u8;
    ERR_VERIFY = 0x32u8;
    ERR_RESOURCE_TEMPORARY_NOT_ACCESSIBLE = 0x33u8;
    
    MAP_ERROR_CODE_TO_NAME = containers.Map( ...
        { 0x00u8, ...
        0x10u8, ...
        0x11u8, ...
        0x12u8, ...
        0x20u8, ...
        0x21u8, ...
        0x22u8, ...
        0x23u8, ...
        0x24u8, ...
        0x25u8, ...
        0x26u8, ...
        0x27u8, ...
        0x28u8, ...
        0x29u8, ...
        0x2Au8, ...
        0x30u8, ...
        0x31u8, ...
        0x32u8, ...
        0x33u8 }, ...
        { 'ERR_CMD_SYNCH', ...
        'ERR_CMD_BUSY', ...
        'ERR_DAQ_ACTIVE', ...
        'ERR_PGM_ACTIVE', ...
        'ERR_CMD_UNKNOWN', ...
        'ERR_CMD_SYNTAX', ...
        'ERR_OUT_OF_RANGE', ...
        'ERR_WRITE_PROTECTED', ...
        'ERR_ACCESS_DENIED', ...
        'ERR_ACCESS_LOCKED', ...
        'ERR_PAGE_NOT_VALID', ...
        'ERR_MODE_NOT_VALID', ...
        'ERR_SEGMENT_NOT_VALID', ...
        'ERR_SEQUENCE', ...
        'ERR_DAQ_CONFIG', ...
        'ERR_MEMORY_OVERFLOW', ...
        'ERR_GENERIC', ...
        'ERR_VERIFY', ...
        'ERR_RESOURCE_TEMPORARY_NOT_ACCESSIBLE' } );

    MAP_ERROR_CODE_TO_EXPLANATION = containers.Map( ...
        { 0x00u8, ...
        0x10u8, ...
        0x11u8, ...
        0x12u8, ...
        0x20u8, ...
        0x21u8, ...
        0x22u8, ...
        0x23u8, ...
        0x24u8, ...
        0x25u8, ...
        0x26u8, ...
        0x27u8, ...
        0x28u8, ...
        0x29u8, ...
        0x2Au8, ...
        0x30u8, ...
        0x31u8, ...
        0x32u8, ...
        0x33u8 }, ...
        { 'Command processor synchronization error.', ...
        'Command was not executed.', ...
        'Command was rejected because DAQ is running.', ...
        'Command was rejected because PGM is running.', ...
        'Unknown command or not implemented optional command.', ...
        'Command syntax is invalid.', ...
        'Command syntax was valid, but command paramater(s) are out of range.', ...
        'The memory location is write protected.', ...
        'The memory location is not accessible.', ...
        'Access denied because a seed and key unlock is required.', ...
        'The selected page is not available.', ...
        'The selected page mode is not available.', ...
        'The selected segment is not valid.', ...
        'A sequence error occurred.', ...
        'DAQ configuration is not valid.', ...
        'A memory overflow error occurred.', ...
        'A generic error occurred.', ...
        'The server internal program verify routine detected an error.', ...
        'Access to the requested resource is temporarily not possible.' } );
end


methods (Static)
   
    function byteArray = packParameter(byteOrder, numberOfBytes, value)
    % packParameter Translates a parameter from decimal value to raw bytes.
    %
    %   This method is used to convert a parameter from a decimal value to
    %   a raw array of bytes for transmission in an XCP command message.
    %   Some command parameters are larger than a single byte and so the
    %   value requires proper handling to transmit. All parameters are
    %   unsigned integer types, so no special conversion handling is
    %   required.
        
        % Convert the input value to a properly sized hexadecimal string.
        switch(numberOfBytes)
            case xcp.Utility.BYTES_IN_BYTE
                d = uint8(value);
            case xcp.Utility.BYTES_IN_WORD
                d = uint16(value);
            case xcp.Utility.BYTES_IN_DWORD
                d = uint32(value);
            case xcp.Utility.BYTES_IN_DLONG
                d = uint64(value);
        end
        
        byteArray = typecast(d, 'uint8');
        
        switch(byteOrder)
            case {'MSB_LAST', 'BYTE_ORDER_MSB_LAST'}
                % This is already in order
            case {'MSB_FIRST', 'BYTE_ORDER_MSB_FIRST'}
                % Swap the bytes
                byteArray = byteArray(numberOfBytes:-1:1);
        end
    end
    
    function value = unpackParameter(byteOrder, numberOfBytes, byteArray)
    % unpackParameter Translates a parameter from raw bytes to decimal value.
    %
    %   This method is used to convert a parameter from a raw array of
    %   bytes to a decimal value after being received in an XCP response
    %   message. Some command parameters are larger than a single byte and
    %   so bytes require proper handling to use. All parameters are
    %   unsigned integer types, so no special conversion handling is
    %   required.
        
        switch(byteOrder)
            case {'MSB_LAST', 'BYTE_ORDER_MSB_LAST'}
                % This is already in order
            case {'MSB_FIRST', 'BYTE_ORDER_MSB_FIRST'}
                % Swap the bytes
                byteArray = byteArray(numberOfBytes:-1:1);
        end
        
        switch(numberOfBytes)
            case xcp.Utility.BYTES_IN_BYTE
                value = typecast(byteArray, 'uint8');
            case xcp.Utility.BYTES_IN_WORD
                value = typecast(byteArray, 'uint16');
            case xcp.Utility.BYTES_IN_DWORD
                value = typecast(byteArray, 'uint32');
            case xcp.Utility.BYTES_IN_DLONG
                value = typecast(byteArray, 'uint64');
        end
        
        value = double(value);
    end
    
    
    function byteArray = packMeasurement(info, value)
    % packMeasurement Translates a measurement from decimal value to raw bytes.
    %
    %   This method is used a measurement is to be transmitted and the
    %   decimal value needs to be converted into a raw array of bytes. The
    %   method enforces engineering unit value limits from the A2L file. It
    %   then applies proper type casting and management to keep the
    %   integrity of the original value. Lastly, the array of bytes are
    %   properly order according to the A2L information.
    
        % Apply minimum and maximum limits.
        if value < info.LowerLimit
            value = info.LowerLimit;
        elseif value > info.UpperLimit
            value = info.UpperLimit;
        end
        
        % Get the data type and count of elements
        [dt, tsize] = xcp.A2L.getMATLABType(info.LocDataType);
        
        % Cast to uint8
        byteArray = typecast(cast(value, dt), 'uint8');
        
        % Change endianness if required
        switch(info.ByteOrder)
            case {'MSB_LAST', 'BYTE_ORDER_MSB_LAST'}
                % This is already in order
            case {'MSB_FIRST', 'BYTE_ORDER_MSB_FIRST'}
                % Swap the bytes
                count = length(value);
                x = repmat(tsize:-1:1,1,count)+repelem(0:tsize:tsize*count-tsize, tsize);
                byteArray = byteArray(x);
        end
    end
    
    function value = unpackMeasurement(info, byteArray)
    % unpackMeasurement Translates a measurement from raw bytes to decimal value.
    %
    %   This method is used when a measurement is received from server and
    %   the raw array of bytes needs to be converted into a decimal value.
    %   The method applies proper type casting and management as well as
    %   enforcing engineering unit value limits from the A2L file.

        % Get the data type and count of elements
        [dt, tsize] = xcp.A2L.getMATLABType(info.LocDataType);
        count = length(byteArray)/tsize;
        % Change endianness if required
        switch(info.ByteOrder)
            case {'MSB_LAST', 'BYTE_ORDER_MSB_LAST'}
                % This is already in order
            case {'MSB_FIRST', 'BYTE_ORDER_MSB_FIRST'}
                % Swap the bytes
                x = repmat(tsize:-1:1,1,count)+repelem(0:tsize:tsize*count-tsize, tsize);
                byteArray = byteArray(x);
        end
        
        % Type cast the value to the appropriate type as a reinterpretive conversion.
        value = typecast(byteArray, dt);

        % Apply minimum and maximum limits.
        if value < info.LowerLimit
            value = info.LowerLimit;
        elseif value > info.UpperLimit
            value = info.UpperLimit;
        end
    end
    
   
end
    
end
